@@ -13,7 +13,7 @@
 setGeneric("truenames", function(x) standardGeneric("truenames"))
 
 setMethod("truenames", signature(x="genind"), function(x){
-  
+
   X <- x@tab
   if(!all(x@ind.names=="")) {rownames(X) <- x@ind.names}
 
@@ -59,17 +59,17 @@ setMethod("truenames",signature(x="genpop"), function(x){
 setGeneric("seploc", function(x, ...) standardGeneric("seploc"))
 
 setMethod("seploc", signature(x="genind"), function(x,truenames=TRUE,res.type=c("genind","matrix")){
-  
+
   if(!is.genind(x)) stop("x is not a valid genind object")
   res.type <- match.arg(res.type)
   if(res.type=="genind") { truenames <- TRUE }
-  
+
   temp <- x@loc.fac
   nloc <- length(levels(temp))
   levels(temp) <- 1:nloc
 
   kX <- list()
-  
+
   for(i in 1:nloc){
     kX[[i]] <- matrix(x@tab[,temp==i],ncol=x@loc.nall[i])
 
@@ -95,8 +95,8 @@ setMethod("seploc", signature(x="genind"), function(x,truenames=TRUE,res.type=c(
           kX[[i]]@other <- x@other
       }
   }
-  
-  return(kX)  
+
+  return(kX)
 })
 
 
@@ -105,17 +105,17 @@ setMethod("seploc", signature(x="genind"), function(x,truenames=TRUE,res.type=c(
 # Method seploc for genpop
 ###########################
 setMethod("seploc", signature(x="genpop"), function(x,truenames=TRUE,res.type=c("genpop","matrix")){
-  
+
   if(!is.genpop(x)) stop("x is not a valid genpop object")
   res.type <- match.arg(res.type)
   if(res.type=="genpop") { truenames <- TRUE }
- 
+
   temp <- x@loc.fac
   nloc <- length(levels(temp))
   levels(temp) <- 1:nloc
 
   kX <- list()
-  
+
   for(i in 1:nloc){
     kX[[i]] <- matrix(x@tab[,temp==i],ncol=x@loc.nall[i])
 
@@ -142,7 +142,7 @@ setMethod("seploc", signature(x="genpop"), function(x,truenames=TRUE,res.type=c(
       }
   }
 
-  return(kX)  
+  return(kX)
 })
 
 
@@ -152,13 +152,38 @@ setMethod("seploc", signature(x="genpop"), function(x,truenames=TRUE,res.type=c(
 # Function adegenetWeb
 #######################
 adegenetWeb <- function(){
-  cat("Opening url \"http://adegenet.r-forge.r-project.org/\" ...\n")
-  browseURL("http://adegenet.r-forge.r-project.org/")
+    cat("Opening url \"http://adegenet.r-forge.r-project.org/\" ...\n")
+    browseURL("http://adegenet.r-forge.r-project.org/")
 }
 
 
 
 
+############################
+# Function adegenetTutorial
+############################
+adegenetTutorial <- function(which=c("general","spca")){
+    which <- match.arg(which)
+    if(which=="general"){
+        url <- "http://adegenet.r-forge.r-project.org/files/adegenet.pdf"
+        cat("\n")
+        cat("  >> Seeking the general tutorial for adegenet.\n")
+        cat("  >> Opening url \"",url,"\".\n ", sep="")
+        cat("\n")
+        browseURL(url)
+    }
+    if(which=="spca"){
+        url <- "http://adegenet.r-forge.r-project.org/files/tutorial-spca.pdf"
+        cat("\n")
+        cat("  >> Seeking the sPCA tutorial for adegenet.\n")
+        cat("  >> Opening url \"",url,"\". \n", sep="")
+        cat("\n")
+        browseURL(url)
+    }
+}
+
+
+
 
 ###############
 # '$' operator
@@ -193,7 +218,7 @@ setMethod("$<-","genpop",function(x,name,value) {
 ###############
 ## genind
 setMethod("[","genind",
-          function(x, i, j, ..., treatOther=TRUE, drop=FALSE) {
+          function(x, i, j, ..., loc=NULL, treatOther=TRUE, drop=FALSE) {
 
               if (missing(i)) i <- TRUE
               if (missing(j)) j <- TRUE
@@ -206,10 +231,20 @@ setMethod("[","genind",
                   pop <- temp$pop
                   pop <- factor(pop[i])
               }
-             
+
+              ## handle loc argument
+              if(!is.null(loc)){
+                  loc <- as.character(loc)
+                  temp <- !loc %in% x@loc.fac
+                  if(any(temp)) { # si mauvais loci
+                      warning(paste("the following specified loci do not exist:", loc[temp]))
+                  }
+                  j <- x$loc.fac %in% loc
+              } # end loc argument
+
               prevcall <- match.call()
               tab <- tab[i, j, ...,drop=FALSE]
-              
+
               res <- genind(tab,pop=pop,prevcall=prevcall)
 
               ## handle 'other' slot
@@ -230,25 +265,35 @@ setMethod("[","genind",
                   } # end f1
 
                   res@other <- lapply(x@other, f1) # treat all elements
-                  
+
               } # end treatOther
-              
+
               return(res)
           })
 
 
 ## genpop
-setMethod("[","genpop", 
-          function(x, i, j, ..., treatOther=TRUE, drop=FALSE) {
+setMethod("[","genpop",
+          function(x, i, j, ..., loc=NULL, treatOther=TRUE, drop=FALSE) {
 
               if (missing(i)) i <- TRUE
               if (missing(j)) j <- TRUE
 
-              tab <- truenames(x) 
-             
+              tab <- truenames(x)
+
+              ## handle loc argument
+              if(!is.null(loc)){
+                  loc <- as.character(loc)
+                  temp <- !loc %in% x@loc.fac
+                  if(any(temp)) { # si mauvais loci
+                      warning(paste("the following specified loci do not exist:", loc[temp]))
+                  }
+                  j <- x$loc.fac %in% loc
+              } # end loc argument
+
               prevcall <- match.call()
               tab <- tab[i, j, ...,drop=FALSE]
-              
+
               res <- genpop(tab,prevcall=prevcall)
 
               ## handle 'other' slot
@@ -264,15 +309,15 @@ setMethod("[","genpop",
                           obj <- obj[i]
                           if(is.factor(obj)) {obj <- factor(obj)}
                       } else {warning(paste("cannot treat the object",namesOther[counter]))}
-                      
+
                       return(obj)
                   } # end f1
-                  
+
                   res@other <- lapply(x@other, f1) # treat all elements
-                  
+
               } # end treatOther
-             
-              
+
+
               return(res)
           })
 
@@ -289,15 +334,19 @@ setGeneric("seppop", function(x, ...) standardGeneric("seppop"))
 ## genind
 setMethod("seppop", signature(x="genind"), function(x,pop=NULL,truenames=TRUE,res.type=c("genind","matrix")){
 
-    ## misc checks 
+    ## misc checks
     if(!is.genind(x)) stop("x is not a valid genind object")
-    if(is.null(pop)) {pop <- x@pop}
+    if(is.null(pop)) { # pop taken from @pop
+        pop <- x@pop
+        levels(pop) <- x@pop.names
+    }
+
     if(is.null(pop)) stop("pop not provided and x@pop is empty")
+
     res.type <- match.arg(res.type)
     if(res.type=="genind") { truenames <- TRUE }
-  
-    pop <- x@pop
-    levels(pop) <- x@pop.names
+
+    ## pop <- x@pop # comment to take pop arg into account
 
     ## make a list of genind objects
     kObj <- lapply(levels(pop), function(lev) x[pop==lev, ])
@@ -305,15 +354,15 @@ setMethod("seppop", signature(x="genind"), function(x,pop=NULL,truenames=TRUE,re
 
     ## res is a list of genind
     if(res.type=="genind"){ return(kObj) }
-  
+
     ## res is list of matrices
     if(truenames) {
         res <- lapply(kObj, function(obj) truenames(obj)$tab)
     } else{
         res <- lapply(kObj, function(obj) obj$tab)
     }
-    
-    return(res) 
+
+    return(res)
 }) # end seppop
 
 
@@ -338,7 +387,7 @@ setMethod("na.replace", signature(x="genind"), function(x,method, quiet=FALSE){
     method <- match.arg(method, c("0","mean"))
 
     res <- x
-    
+
     if(method=="0"){
         res@tab[is.na(x@tab)] <- 0
     }
@@ -379,7 +428,7 @@ setMethod("na.replace", signature(x="genpop"), function(x,method, quiet=FALSE){
     method <- match.arg(method, c("0","chi2"))
 
     res <- x
-    
+
     if(method=="0"){
         res@tab[is.na(x@tab)] <- 0
     }
@@ -422,7 +471,7 @@ repool <- function(...){
     if(!all(table(temp)==length(x))) stop("markers are not the same for all objects")
     temp <- sapply(x,function(e) e$ploidy)
     if(length(unique(temp)) != as.integer(1)) stop("objects have different levels of ploidy")
-    
+
     ## extract info
     listTab <- lapply(x,genind2df,usepop=FALSE)
     getPop <- function(obj){
@@ -431,25 +480,25 @@ repool <- function(...){
         levels(pop) <- obj$pop.names
         return(pop)
     }
-    
+
     ## handle pop
     listPop <- lapply(x, getPop)
     pop <- unlist(listPop, use.name=FALSE)
     pop <- factor(pop)
-    
+
   ## handle genotypes
     markNames <- colnames(listTab[[1]])
     listTab <- lapply(listTab, function(tab) tab[,markNames]) # resorting of the tabs
-    
+
     ## bind all tabs by rows
-    tab <- listTab[[1]] 
+    tab <- listTab[[1]]
     for(i in 2:length(x)){
         tab <- rbind(tab,listTab[[i]])
     }
-    
+
     res <- df2genind(tab,pop=pop)
     res$call <- match.call()
-    
+
     return(res)
 } # end repool
 
