@@ -1,7 +1,7 @@
 ####################
 # Function makefreq
 ####################
-makefreq <- function(x,quiet=FALSE,missing=NA){
+makefreq <- function(x,quiet=FALSE,missing=NA,truenames=TRUE){
 
   if(!is.genpop(x)) stop("x is not a valid genpop object")
 
@@ -9,15 +9,15 @@ makefreq <- function(x,quiet=FALSE,missing=NA){
 
   f1 <- function(v){
     if(all(is.na(v)) || sum(v,na.rm=TRUE)==0) return(rep(NA,length(v)))
-    return(v/(sum(v,na.rm=TRUE)))       
+    return(v/(sum(v,na.rm=TRUE)))
   }
 
   res <- list()
-  
+
   tabcount <- x@tab
-  
-  eff.pop <- t(apply(tabcount,1,function(r) tapply(r,x@loc.fac,sum,na.rm=TRUE)))  
-   
+
+  eff.pop <- t(apply(tabcount,1,function(r) tapply(r,x@loc.fac,sum,na.rm=TRUE)))
+
   # tabfreq is a pop x loci table of allelic frequencies
   tabfreq <- t(apply(tabcount,1,function(r) unlist(tapply(r,x@loc.fac,f1))))
   colnames(tabfreq) <- colnames(x@tab)
@@ -29,8 +29,8 @@ makefreq <- function(x,quiet=FALSE,missing=NA){
     if(toupper(missing)=="MEAN") {
       moy <- apply(tabfreq,2,function(c) mean(c,na.rm=TRUE))
       for(j in 1:ncol(tabfreq)) {tabfreq[,j][is.na(tabfreq[,j])] <- moy[j]}
-    }        
-  }  
+    }
+  }
 
   if(!quiet) cat("\n...done.\n\n")
 
@@ -38,6 +38,16 @@ makefreq <- function(x,quiet=FALSE,missing=NA){
   res$nobs <- eff.pop
   res$call <- match.call()
 
+  ## handle truenames
+  if(truenames){
+      temp <- rep(x@loc.names,x@loc.nall)
+      colnames(res$tab) <- paste(temp,unlist(x@all.names),sep=".")
+      rownames(res$tab) <- x@pop.names
+
+      colnames(res$nobs) <- x@loc.names
+      rownames(res$nobs) <- x@pop.names
+  }
+
   return(res)
 } #end makefreq
 
