@@ -6,7 +6,7 @@
 #
 # generic functions were derived from
 # those of multispati class (ade4)
-# 
+#
 # T. Jombart (jombart@biomserv.univ-lyon1.fr)
 # 31 may 2007
 ##############################################
@@ -16,93 +16,112 @@
 ################
 # Function spca
 ################
-spca <- function(obj, xy=NULL, cn=NULL, scale=FALSE, scannf=TRUE, nfposi=1, nfnega=1, type=NULL,
-                 ask=TRUE, plot.nb=TRUE, edit.nb=FALSE ,truenames=TRUE, d1=NULL, d2=NULL, k=NULL,
-                 a=NULL, dmin=NULL) {
-  
-  if(!any(inherits(obj,c("genind","genpop")))) stop("obj must be a genind or genpop object.")
-  invisible(validObject(obj))
+spca <- function(obj, xy=NULL, cn=NULL, matWeight=NULL,
+                 scale=FALSE, scale.method=c("sigma","binom"),
+                 scannf=TRUE, nfposi=1, nfnega=1,
+                 type=NULL, ask=TRUE, plot.nb=TRUE, edit.nb=FALSE,
+                 truenames=TRUE, d1=NULL, d2=NULL, k=NULL, a=NULL, dmin=NULL){
 
-  ## spatial coordinates
-  if(is.null(xy) & !is.null(obj$other$xy)) xy <- obj$other$xy
-  if(is.data.frame(xy)) xy <- as.matrix(xy)
-  if(!is.null(xy) & !is.matrix(xy)) stop("wrong 'xy' provided")
-  
-  if(!require(ade4, quiet=TRUE)) stop("ade4 library is required.")
+    ## first checks
+    if(!any(inherits(obj,c("genind","genpop")))) stop("obj must be a genind or genpop object.")
+    invisible(validObject(obj))
+    if(!require(ade4, quiet=TRUE)) stop("ade4 library is required.")
+    if(!require(spdep, quiet=TRUE)) stop("spdep library is required.")
 
-  appel <- match.call()
-  
-  ## connection network
-  if(is.null(cn)) {
-    if(is.null(xy)) stop("'xy' and 'cn' are both missing")
-    resCN <- chooseCN(xy=xy, ask=ask, type=type, plot.nb=plot.nb, edit.nb=edit.nb,
-                      result.type="listw", d1=d1, d2=d2, k=k, a=a, dmin=dmin)
-  } else {
-      if(inherits(cn,"nb") & !inherits(cn,"listw")) {
-          xy <- attr(cn,"xy") # xy coords can be retrieved from cn of class nb (not from listw) 
-          cn <- nb2listw(cn, style="W", zero.policy=TRUE)
-      }
+    ## handle xy coordinates
+    if(is.null(xy) & (inherits(cn,"nb") & !inherits(cn,"listw")) ){
+                     xy <- attr(cn,"xy")  # xy can be retrieved from a nb object (not from listw)
+                 }
+    if(is.null(xy) & !is.null(obj$other$xy)) {
+        xy <- obj$other$xy # xy from @other$xy if it exists
+    }
+    if(is.null(xy)) stop("xy coordinates are not provided")
+    if(is.data.frame(xy)) {
+        xy <- as.matrix(xy)
+    }
+    if(!is.matrix(xy)) stop("wrong 'xy' provided")
+    if(ncol(xy) != 2) stop("xy does not have two columns.")
+    if(nrow(xy) != nrow(obj@tab)) stop("obj@tab and xy must have the same row numbers.")
 
-      if(!inherits(cn,"listw")) {
-          stop("cn does not have a recognized class ('nb' or 'listw', package spdep)")
-      } else {
-          if(is.null(xy)) stop("listw object provided as 'cn' without providing 'xy'")
-          resCN <- cn
-      }
-  }
 
-  ## check xy coordinates
-  if(ncol(xy) != 2) stop("xy does not have two columns.")
-  if(nrow(xy) != nrow(obj@tab)) stop("obj@tab and xy must have the same row numbers.")
+    appel <- match.call()
 
-  ## prepare data
-  f1 <- function(vec){
-    m <- mean(vec,na.rm=TRUE)
-    vec[is.na(vec)] <- m
-    return(vec)
-  }
+    ## == To find the spatial weights ==
+    resCN <- NULL
 
-  if(is.genind(obj)) { X <- obj@tab }
-  if(is.genpop(obj)) { X <- makefreq(obj, quiet=TRUE)$tab }
+    ## connection network from matWeight
+    if(!is.null(matWeight)){
+        if(!is.matrix(matWeight)) stop("matWeight is not a matrix")
+        if(!is.numeric(matWeight)) stop("matWeight is not numeric")
+        if(nrow(matWeight) != ncol(matWeight)) stop("matWeight is not square")
+        if(nrow(matWeight) != nrow(obj@tab)) stop("dimension of datWeight does not match genetic data")
+        diag(matWeight) <- 0
+        matWeight <- prop.table(matWeight, 1)
+        resCN <- mat2listw(matWeight)
+        resCN$style <- "W"
 
-  ## handle NAs
-  if(any(is.na(X))){
-      warning("NAs in data are automatically replaced (to mean allele frequency")
-      X <- apply(X,2,f1)
-  }
+    }
 
-  if(truenames){
-    rownames(X) <- rownames(truenames(obj))
-    colnames(X) <- colnames(truenames(obj))   
-  }
+    ## connection network from cn argument
+    if(is.null(resCN) & !is.null(cn)) {
+        if(inherits(cn,"nb")) {
+            if(!inherits(cn,"listw")){ # cn is a 'pure' nb object (i.e., nb but not listw)
+                cn <- nb2listw(cn, style="W", zero.policy=TRUE)
+            }
+            resCN <- cn
+        } else stop("cn does not have a recognized class")
+    }
 
-  # perform analyses
-  pcaX <- dudi.pca(X, center=TRUE, scale=scale, scannf=FALSE)
 
-  spcaX <- multispati(dudi=pcaX, listw=resCN, scannf=scannf, nfposi=nfposi, nfnega=nfnega)
+    ## connection network from xy coordinates
+    if(is.null(resCN)) {
+        resCN <- chooseCN(xy=xy, ask=ask, type=type, plot.nb=plot.nb, edit.nb=edit.nb,
+                          result.type="listw", d1=d1, d2=d2, k=k, a=a, dmin=dmin)
+    }
 
-  nfposi <- spcaX$nfposi
-  nfnega <- spcaX$nfnega
+    ## == spatial weights are done ==
 
-  spcaX$xy <- xy
-  rownames(spcaX$xy) <- rownames(spcaX$li)
-  colnames(spcaX$xy) <- c("x","y")
-  
-  spcaX$lw <- resCN
-  
-  spcaX$call <- appel
 
-  posaxes <- if(nfposi>0) {1:nfposi} else NULL
-  negaxes <- if(nfnega>0) {(length(spcaX$eig)-nfnega+1):length(spcaX$eig)} else NULL
-  keptaxes <- c(posaxes,negaxes)
-      
-  colnames(spcaX$c1) <- paste("Axis",keptaxes)
-  colnames(spcaX$li) <- paste("Axis",keptaxes)
-  colnames(spcaX$ls) <- paste("Axis",keptaxes)
+    ## handle NAs warning
+    if(any(is.na(obj@tab))){
+        warning("NAs in data are automatically replaced (to mean allele frequency")
+    }
 
-  class(spcaX) <- "spca"
+    ## handle NAs, centring and scaling
+    X <- scaleGen(obj, center=TRUE, scale=scale, method=scale.method, missing="mean", truenames=truenames)
 
-  return(spcaX)
+    ## perform analyses
+    pcaX <- dudi.pca(X, center=FALSE, scale=FALSE, scannf=FALSE)
+
+    spcaX <- multispati(dudi=pcaX, listw=resCN, scannf=scannf, nfposi=nfposi, nfnega=nfnega)
+
+    nfposi <- spcaX$nfposi
+    nfnega <- spcaX$nfnega
+
+    spcaX$xy <- xy
+    rownames(spcaX$xy) <- rownames(spcaX$li)
+    colnames(spcaX$xy) <- c("x","y")
+
+    spcaX$lw <- resCN
+
+    spcaX$call <- appel
+
+    posaxes <- if(nfposi>0) {1:nfposi} else NULL
+    negaxes <- if(nfnega>0) {(length(spcaX$eig)-nfnega+1):length(spcaX$eig)} else NULL
+    keptaxes <- c(posaxes,negaxes)
+
+    ## set names of different components
+    colnames(spcaX$c1) <- paste("Axis",keptaxes)
+    colnames(spcaX$li) <- paste("Axis",keptaxes)
+    colnames(spcaX$ls) <- paste("Axis",keptaxes)
+    row.names(spcaX$c1) <- colnames(X)
+    colnames(spcaX$as) <- colnames(spcaX$c1)
+    temp <- row.names(spcaX$as)
+    row.names(spcaX$as) <- paste("PCA",temp)
+
+    class(spcaX) <- "spca"
+
+    return(spcaX)
 
 } # end spca
 
@@ -115,7 +134,7 @@ spca <- function(obj, xy=NULL, cn=NULL, scale=FALSE, scannf=TRUE, nfposi=1, nfne
 ######################
 print.spca <- function(x, ...){
   cat("\t########################################\n")
-  cat("\t# Spatial principal component analysis #\n")
+  cat("\t# spatial Principal Component Analysis #\n")
   cat("\t########################################\n")
   cat("class: ")
   cat(class(x))
@@ -123,21 +142,21 @@ print.spca <- function(x, ...){
   print(x$call)
   cat("\n$nfposi:", x$nfposi, "axis-components saved")
   cat("\n$nfnega:", x$nfnega, "axis-components saved")
- 
+
   cat("\nPositive eigenvalues: ")
   l0 <- sum(x$eig >= 0)
   cat(signif(x$eig, 4)[1:(min(5, l0))])
-  if (l0 > 5) 
+  if (l0 > 5)
     cat(" ...\n")
-  else cat("\n")  
+  else cat("\n")
   cat("Negative eigenvalues: ")
   l0 <- sum(x$eig <= 0)
   cat(sort(signif(x$eig, 4))[1:(min(5, l0))])
-  if (l0 > 5) 
+  if (l0 > 5)
     cat(" ...\n")
   else cat("\n")
   cat('\n')
-  sumry <- array("", c(1, 4), list(1, c("vector", "length", 
+  sumry <- array("", c(1, 4), list(1, c("vector", "length",
                                         "mode", "content")))
   sumry[1, ] <- c('$eig', length(x$eig), mode(x$eig), 'eigenvalues')
   class(sumry) <- "table"
@@ -145,18 +164,18 @@ print.spca <- function(x, ...){
   cat("\n")
   sumry <- array("", c(4, 4), list(1:4, c("data.frame", "nrow", "ncol", "content")))
   sumry[1, ] <- c("$c1", nrow(x$c1), ncol(x$c1), "principal axes: scaled vectors of alleles loadings")
-  sumry[2, ] <- c("$li", nrow(x$li), ncol(x$li), "principal components: coordinates of entities")
+  sumry[2, ] <- c("$li", nrow(x$li), ncol(x$li), "principal components: coordinates of entities ('scores')")
   sumry[3, ] <- c("$ls", nrow(x$ls), ncol(x$ls), 'lag vector of principal components')
   sumry[4, ] <- c("$as", nrow(x$as), ncol(x$as), 'pca axes onto spca axes')
-  
+
   class(sumry) <- "table"
   print(sumry)
 
   cat("\n$xy: matrix of spatial coordinates")
   cat("\n$lw: a list of spatial weights (class 'listw')")
-  
+
   cat("\n\nother elements: ")
-  if (length(names(x)) > 10) 
+  if (length(names(x)) > 10)
     cat(names(x)[11:(length(names(x)))], "\n")
   else cat("NULL\n")
 }
@@ -185,41 +204,41 @@ summary.spca <- function (object, ..., printres=TRUE) {
   }
 
   resfin <- list()
-  
+
   if(printres) {
     cat("\nSpatial principal component analysis\n")
     cat("\nCall: ")
     print(object$call)
   }
-  
+
   appel <- as.list(object$call)
   ## compute original pca
   # prepare data
   obj <- eval(appel$obj)
   if(is.null(appel$truenames)) truenames <- FALSE
-  
+
   f1 <- function(vec){
     m <- mean(vec,na.rm=TRUE)
     vec[is.na(vec)] <- m
     return(vec)
   }
-  
+
   if(is.genind(obj)) { X <- obj@tab }
   if(is.genpop(obj)) { X <- makefreq(obj, quiet=TRUE)$tab }
-  
+
   X <- apply(X,2,f1)
-  
+
   if(truenames){
     rownames(X) <- rownames(truenames(obj))
-    colnames(X) <- colnames(truenames(obj))   
+    colnames(X) <- colnames(truenames(obj))
   }
-  
+
   nfposi <- object$nfposi
   nfnega <- object$nfnega
-  
+
   dudi <- dudi.pca(X, center=TRUE, scale=FALSE, scannf=FALSE, nf=nfposi+nfnega)
   ## end of pca
-    
+
   lw <- object$lw
 
   # I0, Imin, Imax
@@ -240,7 +259,7 @@ summary.spca <- function (object, ..., printres=TRUE) {
   names(Istat) <- c("I0","Imin","Imax")
   resfin$Istat <- Istat
 
-  
+
   # les scores de l'analyse de base
   nf <- dudi$nf
   eig <- dudi$eig[1:nf]
@@ -257,13 +276,13 @@ summary.spca <- function (object, ..., printres=TRUE) {
 
   resfin$pca <- res
 
-  
+
   # les scores de l'analyse spatiale
   # on recalcule l'objet en gardant tous les axes
   eig <- object$eig
   nfposimax <- sum(eig > 0)
   nfnegamax <- sum(eig < 0)
-    
+
   ms <- multispati(dudi=dudi, listw=lw, scannf=FALSE,
                      nfposi=nfposimax, nfnega=nfnegamax)
 
@@ -274,14 +293,14 @@ summary.spca <- function (object, ..., printres=TRUE) {
   moran <- apply(as.matrix(ms$li)*as.matrix(ms$ls)*dudi$lw,2,sum)
   res <- data.frame(eig=eig,var=varspa,moran=moran/varspa)
   row.names(res) <- paste("Axis",1:length(eig))
-  
+
   if(printres) {
     cat("\nsPCA eigenvalues decomposition:\n")
     print(res[agarder,])
   }
-  
+
   resfin$spca <- res
-    
+
   return(invisible(resfin))
 }
 
@@ -290,20 +309,26 @@ summary.spca <- function (object, ..., printres=TRUE) {
 #####################
 # Function plot.spca
 #####################
-plot.spca <- function (x, axis = 1, ...){
+plot.spca <- function (x, axis = 1, useLag=FALSE, ...){
     if (!inherits(x, "spca")) stop("Use only with 'spca' objects.")
 
     if(!require(ade4)) stop("ade4 package is required.")
     if(!require(spdep)) stop("spdep package is required.")
     if(axis>ncol(x$li)) stop("wrong axis required.")
-    
+
     opar <- par(no.readonly = TRUE)
     on.exit(par(opar))
     par(mar = rep(.1,4), mfrow=c(3,2))
 
     n <- nrow(x$li)
     xy <- x$xy
-    z <- x$ls[,axis]
+
+    ## handle useLag argument
+    if(useLag){
+        z <- x$ls[,axis]
+    } else {
+        z <- x$li[,axis]
+    } # end if useLag
     nfposi <- x$nfposi
     nfnega <- x$nfnega
     ## handle neig parameter - hide cn if nore than 100 links
@@ -313,29 +338,29 @@ plot.spca <- function (x, axis = 1, ...){
     } else {
         neig <- NULL
     }
-    
+
     sub <- paste("Score",axis)
     csub <- 2
-      
+
     # 1
     if(n<30) clab <- 1 else clab <- 0
     s.label(xy, clab=clab, include.ori=FALSE, addaxes=FALSE, neig=neig,
-            cneig=1, sub="Connection network", csub=2)    
-    
+            cneig=1, sub="Connection network", csub=2)
+
     # 2
     s.image(xy,z, include.ori=FALSE, grid=TRUE, kgrid=10, cgrid=1,
             sub=sub, csub=csub, possub="bottomleft")
     box()
-    
+
     # 3
     if(n<30) {neig <- nb2neig(x$lw$neighbours)} else {neig <- NULL}
     s.value(xy,z, include.ori=FALSE, addaxes=FALSE, clegend=0, csize=.6,
             neig=neig, sub=sub, csub=csub, possub="bottomleft")
-    
+
     # 4
     s.value(xy,z, include.ori=FALSE, addaxes=FALSE, clegend=0, csize=.6,
             method="greylevel", neig=neig, sub=sub, csub=csub, possub="bottomleft")
-        
+
     # 5
     omar <- par("mar")
     par(mar = c(0.8, 2.8, 0.8, 0.8))
@@ -345,13 +370,13 @@ plot.spca <- function (x, axis = 1, ...){
     if (nfnega>0) {col.w[m:(m-nfnega+1)] <- "grey"}
     j <- axis
     if (j>nfposi) {j <- j-nfposi +m -nfnega}
-    col.w[j] <- "black" 
+    col.w[j] <- "black"
     barplot(x$eig, col = col.w)
     scatterutil.sub(cha ="Eigenvalues", csub = 2.5, possub = "topright")
     par(mar=rep(.1,4))
     box()
     par(mar=omar)
-    
+
     # 6
     par(mar=c(4,4,2,1))
     screeplot(x,main="Eigenvalues decomposition")
@@ -371,11 +396,11 @@ screeplot.spca <- function(x,...,main=NULL){
   on.exit(par(las=opar))
 
   sumry <- summary(x,printres=FALSE)
-  
+
   labels <- lapply(1:length(x$eig),function(i) bquote(lambda[.(i)]))
 
   par(las=1)
-  
+
   xmax <- sumry$pca[1,1]*1.1
   I0 <- sumry$Istat[1]
   Imin <- sumry$Istat[2]
@@ -383,7 +408,7 @@ screeplot.spca <- function(x,...,main=NULL){
 
   plot(x=sumry$spca[,2],y=sumry$spca[,3],type='n',xlab='Variance',ylab="Spatial autocorrelation (I)",xlim=c(0,xmax),ylim=c(Imin*1.1,Imax*1.1),yaxt='n',...)
   text(x=sumry$spca[,2],y=sumry$spca[,3],do.call(expression,labels))
-  
+
   ytick <- c(I0,round(seq(Imin,Imax,le=5),1))
   ytlab <- as.character(round(seq(Imin,Imax,le=5),1))
   ytlab <- c(as.character(round(I0,1)),as.character(round(Imin,1)),ytlab[2:4],as.character(round(Imax,1)))
@@ -395,6 +420,31 @@ screeplot.spca <- function(x,...,main=NULL){
 
   if(is.null(main)) main <- ("Spatial and variance components of the eigenvalues")
   title(main)
-  
+
   return(invisible(match.call()))
 }
+
+
+
+
+
+###################
+# colorplot method
+###################
+colorplot.spca <- function(x, axes=1:ncol(x$li), useLag=FALSE, ...){
+    ## some checks
+    if(!any(inherits(x,"spca"))) stop("x in not a spca object.")
+
+    ## get args to be passed to colorplot
+    xy <- x$xy
+
+    if(useLag) {
+        X <- as.matrix(x$ls)
+    } else {
+        X <- as.matrix(x$li)
+    }
+
+    ## call to colorplot
+    colorplot(xy, X, axes, ...)
+
+} # end colorplot.spca
